@@ -1,7 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1991, 1992, 1993, 1994, 2004, 2006, 2007, 2008 Free
-   Software Foundation, Inc.
+   Copyright (C) 1991-1994, 2004, 2006-2007, 2009-2014, 2016-2017,
+   2020-2021 Free Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "m4.h"
@@ -30,7 +30,7 @@ FILE *debug = NULL;
 /* Obstack for trace messages.  */
 static struct obstack trace;
 
-static void debug_set_file (const char *, FILE *);
+static void debug_set_file (FILE *);
 
 /*----------------------------------.
 | Initialise the debugging module.  |
@@ -39,13 +39,13 @@ static void debug_set_file (const char *, FILE *);
 void
 debug_init (void)
 {
-  debug_set_file (NULL, stderr);
+  debug_set_file (stderr);
   obstack_init (&trace);
 }
 
 /*-----------------------------------------------------------------.
 | Function to decode the debugging flags OPTS.  Used by main while |
-| processing option -d, and by the builtin debugmode ().	   |
+| processing option -d, and by the builtin debugmode ().           |
 `-----------------------------------------------------------------*/
 
 int
@@ -58,94 +58,107 @@ debug_decode (const char *opts)
   else
     {
       for (level = 0; *opts; opts++)
-	{
-	  switch (*opts)
-	    {
-	    case 'a':
-	      level |= DEBUG_TRACE_ARGS;
-	      break;
+        {
+          switch (*opts)
+            {
+            case 'a':
+              level |= DEBUG_TRACE_ARGS;
+              break;
 
-	    case 'e':
-	      level |= DEBUG_TRACE_EXPANSION;
-	      break;
+            case 'e':
+              level |= DEBUG_TRACE_EXPANSION;
+              break;
 
-	    case 'q':
-	      level |= DEBUG_TRACE_QUOTE;
-	      break;
+            case 'q':
+              level |= DEBUG_TRACE_QUOTE;
+              break;
 
-	    case 't':
-	      level |= DEBUG_TRACE_ALL;
-	      break;
+            case 't':
+              level |= DEBUG_TRACE_ALL;
+              break;
 
-	    case 'l':
-	      level |= DEBUG_TRACE_LINE;
-	      break;
+            case 'l':
+              level |= DEBUG_TRACE_LINE;
+              break;
 
-	    case 'f':
-	      level |= DEBUG_TRACE_FILE;
-	      break;
+            case 'f':
+              level |= DEBUG_TRACE_FILE;
+              break;
 
-	    case 'p':
-	      level |= DEBUG_TRACE_PATH;
-	      break;
+            case 'p':
+              level |= DEBUG_TRACE_PATH;
+              break;
 
-	    case 'c':
-	      level |= DEBUG_TRACE_CALL;
-	      break;
+            case 'c':
+              level |= DEBUG_TRACE_CALL;
+              break;
 
-	    case 'i':
-	      level |= DEBUG_TRACE_INPUT;
-	      break;
+            case 'i':
+              level |= DEBUG_TRACE_INPUT;
+              break;
 
-	    case 'x':
-	      level |= DEBUG_TRACE_CALLID;
-	      break;
+            case 'x':
+              level |= DEBUG_TRACE_CALLID;
+              break;
 
-	    case 'V':
-	      level |= DEBUG_TRACE_VERBOSE;
-	      break;
+            case 'V':
+              level |= DEBUG_TRACE_VERBOSE;
+              break;
 
-	    default:
-	      return -1;
-	    }
-	}
+            default:
+              return -1;
+            }
+        }
     }
+
+  /* This is to avoid screwing up the trace output due to changes in the
+     debug_level.  */
+
+  obstack_free (&trace, obstack_finish (&trace));
+
   return level;
 }
 
-/*------------------------------------------------------------------------.
-| Change the debug output stream to FP.  If the underlying file is the	  |
-| same as stdout, use stdout instead so that debug messages appear in the |
-| correct relative position.						  |
-`------------------------------------------------------------------------*/
+/*-----------------------------------------------------------------.
+| Change the debug output stream to FP.  If the underlying file is |
+| the same as stdout, use stdout instead so that debug messages    |
+| appear in the correct relative position.                         |
+`-----------------------------------------------------------------*/
 
 static void
-debug_set_file (const char *caller, FILE *fp)
+debug_set_file (FILE *fp)
 {
   struct stat stdout_stat, debug_stat;
 
   if (debug != NULL && debug != stderr && debug != stdout
       && close_stream (debug) != 0)
-    m4_error (0, errno, caller, _("error writing to debug stream"));
+    {
+      M4ERROR ((warning_status, errno, _("error writing to debug stream")));
+      retcode = EXIT_FAILURE;
+    }
   debug = fp;
 
   if (debug != NULL && debug != stdout)
     {
       if (fstat (STDOUT_FILENO, &stdout_stat) < 0)
-	return;
+        return;
       if (fstat (fileno (debug), &debug_stat) < 0)
-	return;
+        return;
 
       /* mingw has a bug where fstat on a regular file reports st_ino
-	 of 0.  On normal system, st_ino should never be 0.  */
+         of 0.  On normal system, st_ino should never be 0.  */
       if (stdout_stat.st_ino == debug_stat.st_ino
-	  && stdout_stat.st_dev == debug_stat.st_dev
-	  && stdout_stat.st_ino != 0)
-	{
-	  if (debug != stderr && close_stream (debug) != 0)
-	    m4_error (0, errno, caller, _("error writing to debug stream"));
-	  debug = stdout;
-	}
+          && stdout_stat.st_dev == debug_stat.st_dev
+          && stdout_stat.st_ino != 0)
+        {
+          if (debug != stderr && close_stream (debug) != 0)
+            {
+              M4ERROR ((warning_status, errno,
+                        _("error writing to debug stream")));
+              retcode = EXIT_FAILURE;
+            }
+          debug = stdout;
+        }
     }
 }
 
@@ -180,38 +193,35 @@ debug_flush_files (void)
     }
 }
 
-/*-------------------------------------------------------------------.
-| Change the debug output to file NAME.  If NAME is NULL, debug	     |
-| output is reverted to stderr, and if empty, debug output is	     |
-| discarded.  Return true iff the output stream was changed.  Report |
-| errors on behalf of CALLER.					     |
-`-------------------------------------------------------------------*/
+/*--------------------------------------------------------------.
+| Change the debug output to file NAME.  If NAME is NULL, debug |
+| output is reverted to stderr, and if empty, debug output is   |
+| discarded.  Return true iff the output stream was changed.    |
+`--------------------------------------------------------------*/
 
 bool
-debug_set_output (const char *caller, const char *name)
+debug_set_output (const char *name)
 {
   FILE *fp;
 
   if (name == NULL)
-    debug_set_file (caller, stderr);
+    debug_set_file (stderr);
   else if (*name == '\0')
-    debug_set_file (caller, NULL);
+    debug_set_file (NULL);
   else
     {
-      fp = fopen (name, "a");
+      fp = fopen (name, "ae");
       if (fp == NULL)
-	return false;
-
-      if (set_cloexec_flag (fileno (fp), true) != 0)
-	m4_warn (errno, caller, _("cannot protect debug file across forks"));
-      debug_set_file (caller, fp);
+        return false;
+      debug_set_file (fp);
     }
   return true;
 }
 
-/*-----------------------------------------------------------------------.
-| Print the header of a one-line debug message, starting by "m4 debug".	 |
-`-----------------------------------------------------------------------*/
+/*--------------------------------------------------------------.
+| Print the header of a one-line debug message, starting by "m4 |
+| debug".                                                       |
+`--------------------------------------------------------------*/
 
 void
 debug_message_prefix (void)
@@ -233,69 +243,69 @@ debug_message_prefix (void)
    output from interfering with other debug messages generated by the
    various builtins.  */
 
-/*-------------------------------------------------------------------.
-| Tracing output to the obstack is formatted here, by a simplified   |
-| printf-like function trace_format ().  Understands only %B (1 arg: |
-| input block), %S (1 arg: length-limited text), %s (1 arg: text),   |
-| %d (1 arg: integer), %l (0 args: optional left quote) and %r (0    |
-| args: optional right quote).                                       |
-`-------------------------------------------------------------------*/
+/*------------------------------------------------------------------.
+| Tracing output is formatted here, by a simplified                 |
+| printf-to-obstack function trace_format ().  Understands only %S, |
+| %s, %d, %l (optional left quote) and %r (optional right quote).   |
+`------------------------------------------------------------------*/
 
 static void
 trace_format (const char *fmt, ...)
 {
   va_list args;
   char ch;
+
   int d;
   const char *s;
-  size_t maxlen;
+  int slen;
+  int maxlen;
 
   va_start (args, fmt);
 
   while (true)
     {
       while ((ch = *fmt++) != '\0' && ch != '%')
-	obstack_1grow (&trace, ch);
+        obstack_1grow (&trace, ch);
 
       if (ch == '\0')
-	break;
+        break;
 
-      maxlen = SIZE_MAX;
+      maxlen = 0;
       switch (*fmt++)
-	{
-	case 'B':
-	  s = "";
-	  input_print (&trace, va_arg (args, input_block *));
-	  break;
+        {
+        case 'S':
+          maxlen = max_debug_argument_length;
+          FALLTHROUGH;
+        case 's':
+          s = va_arg (args, const char *);
+          break;
 
-	case 'S':
-	  maxlen = max_debug_argument_length;
-	  /* fall through */
+        case 'l':
+          s = (debug_level & DEBUG_TRACE_QUOTE) ? lquote.string : "";
+          break;
 
-	case 's':
-	  s = va_arg (args, const char *);
-	  break;
+        case 'r':
+          s = (debug_level & DEBUG_TRACE_QUOTE) ? rquote.string : "";
+          break;
 
-	case 'l':
-	  s = (debug_level & DEBUG_TRACE_QUOTE) ? curr_quote.str1 : "";
-	  break;
+        case 'd':
+          d = va_arg (args, int);
+          s = ntoa (d, 10);
+          break;
 
-	case 'r':
-	  s = (debug_level & DEBUG_TRACE_QUOTE) ? curr_quote.str2 : "";
-	  break;
+        default:
+          s = "";
+          break;
+        }
 
-	case 'd':
-	  d = va_arg (args, int);
-	  s = ntoa (d, 10);
-	  break;
-
-	default:
-	  s = "";
-	  break;
-	}
-
-      if (shipout_string_trunc (&trace, s, SIZE_MAX, &maxlen))
-	break;
+      slen = strlen (s);
+      if (maxlen == 0 || maxlen > slen)
+        obstack_grow (&trace, s, slen);
+      else
+        {
+          obstack_grow (&trace, s, maxlen);
+          obstack_grow (&trace, "...", 3);
+        }
     }
 
   va_end (args);
@@ -303,7 +313,6 @@ trace_format (const char *fmt, ...)
 
 /*------------------------------------------------------------------.
 | Format the standard header attached to all tracing output lines.  |
-| ID is the current macro id.                                       |
 `------------------------------------------------------------------*/
 
 static void
@@ -313,9 +322,9 @@ trace_header (int id)
   if (current_line)
     {
       if (debug_level & DEBUG_TRACE_FILE)
-	trace_format ("%s:", current_file);
+        trace_format ("%s:", current_file);
       if (debug_level & DEBUG_TRACE_LINE)
-	trace_format ("%d:", current_line);
+        trace_format ("%d:", current_line);
     }
   trace_format (" -%d- ", expansion_level);
   if (debug_level & DEBUG_TRACE_CALLID)
@@ -337,10 +346,10 @@ trace_flush (void)
   obstack_free (&trace, line);
 }
 
-/*----------------------------------------------------------------.
-| Do pre-argument-collection tracing for macro NAME, with a given |
-| ID.  Used from expand_macro ().                                 |
-`----------------------------------------------------------------*/
+/*-------------------------------------------------------------.
+| Do pre-argument-collction tracing for macro NAME.  Used from |
+| expand_macro ().                                             |
+`-------------------------------------------------------------*/
 
 void
 trace_prepre (const char *name, int id)
@@ -350,25 +359,54 @@ trace_prepre (const char *name, int id)
   trace_flush ();
 }
 
-/*-----------------------------------------------------------------.
-| Format the parts of a trace line that are known before the macro |
-| is actually expanded.  Called for the macro NAME with ID, and    |
-| arguments ARGV.  Used from expand_macro ().                      |
-`-----------------------------------------------------------------*/
+/*--------------------------------------------------------------.
+| Format the parts of a trace line, that can be made before the |
+| macro is actually expanded.  Used from expand_macro ().       |
+`--------------------------------------------------------------*/
 
 void
-trace_pre (const char *name, int id, macro_arguments *argv)
+trace_pre (const char *name, int id, int argc, token_data **argv)
 {
+  int i;
+  const builtin *bp;
+
   trace_header (id);
   trace_format ("%s", name);
 
-  if (arg_argc (argv) > 1 && (debug_level & DEBUG_TRACE_ARGS))
+  if (argc > 1 && (debug_level & DEBUG_TRACE_ARGS))
     {
-      size_t len = max_debug_argument_length;
       trace_format ("(");
-      arg_print (&trace, argv, 1,
-		 (debug_level & DEBUG_TRACE_QUOTE) ? &curr_quote : NULL,
-		 false, ", ", &len, true);
+
+      for (i = 1; i < argc; i++)
+        {
+          if (i != 1)
+            trace_format (", ");
+
+          switch (TOKEN_DATA_TYPE (argv[i]))
+            {
+            case TOKEN_TEXT:
+              trace_format ("%l%S%r", TOKEN_DATA_TEXT (argv[i]));
+              break;
+
+            case TOKEN_FUNC:
+              bp = find_builtin_by_addr (TOKEN_DATA_FUNC (argv[i]));
+              if (bp == NULL)
+                {
+                  M4ERROR ((warning_status, 0, "\
+INTERNAL ERROR: builtin not found in builtin table! (trace_pre ())"));
+                  abort ();
+                }
+              trace_format ("<%s>", bp->name);
+              break;
+
+            case TOKEN_VOID:
+            default:
+              M4ERROR ((warning_status, 0,
+                        "INTERNAL ERROR: bad token data type (trace_pre ())"));
+              abort ();
+            }
+
+        }
       trace_format (")");
     }
 
@@ -380,17 +418,13 @@ trace_pre (const char *name, int id, macro_arguments *argv)
 }
 
 /*-------------------------------------------------------------------.
-| Format the final part of a trace line and print it all.  Print     |
-| details for macro NAME with ID, given arguemnts ARGV and expansion |
-| EXPANDED.  Used from expand_macro ().                              |
+| Format the final part of a trace line and print it all.  Used from |
+| expand_macro ().                                                   |
 `-------------------------------------------------------------------*/
 
 void
-trace_post (const char *name, int id, macro_arguments *argv,
-	    const input_block *expanded)
+trace_post (const char *name, int id, int argc, const char *expanded)
 {
-  int argc = arg_argc (argv);
-
   if (debug_level & DEBUG_TRACE_CALL)
     {
       trace_header (id);
@@ -398,6 +432,6 @@ trace_post (const char *name, int id, macro_arguments *argv,
     }
 
   if (expanded && (debug_level & DEBUG_TRACE_EXPANSION))
-    trace_format (" -> %l%B%r", expanded);
+    trace_format (" -> %l%S%r", expanded);
   trace_flush ();
 }
